#include <nvidia.h>
#include <d3dframework.h>

//-----------------------------------------------------------------------
// Name: LoadTGAFile()
// Desc: Given a filename of a TGA file, it loads the image in BITMAP form
//-----------------------------------------------------------------------

unsigned char *LoadTGAHeader (FILE * fpin, DWORD * width, DWORD * height, 
                              int swap, int pad);

HRESULT LoadTGAFile( TCHAR* strFilename, HBITMAP* phbm )
{
  unsigned char *data;
  unsigned char *filedata;
    // Check params
    if( NULL==strFilename || NULL==phbm )
        return DDERR_INVALIDPARAMS;

    // Open the file and read the header
    FILE* file = _tfopen( strFilename, TEXT("rb") );
    if( NULL == file )
        return DDERR_NOTFOUND;

    // Parse the TGA header
    DWORD dwWidth, dwHeight;

    data = LoadTGAHeader( file, &dwWidth, &dwHeight,  0, 1);

    if (data == 0)
    {
        fclose( file );
        return DDERR_GENERIC;
    }

    filedata = data;
    // Create a bitmap to load the data into
    LONG*      pBits    = NULL;
    HDC        hdcImage = CreateCompatibleDC(NULL);
    BITMAPINFO bmi      = { sizeof(BITMAPINFOHEADER), dwWidth, dwHeight, 1, 32,
                            BI_RGB, dwWidth*dwHeight*4, 4000, 4000, 0, 0 };

    if( NULL == ( (*phbm) = CreateDIBSection( hdcImage, &bmi, DIB_RGB_COLORS, 
                                              (VOID**)&pBits, NULL, 0 ) ) )
    {
        DeleteDC( hdcImage );
        fclose( file );
        return DDERR_GENERIC; 
    }

    // Read in all the pixels for the texture map
    for( DWORD y=0; y<dwHeight; y++ ) 
    {
        for( DWORD x = 0; x<dwWidth; x++ ) 
        {
            // Read each pixel value and store it in the bitmap bits
            ((BYTE*)pBits)[0] = *data++;
            ((BYTE*)pBits)[1] = *data++; // red component
            ((BYTE*)pBits)[2] = *data++; // green component
            ((BYTE*)pBits)[3] = *data++; // blue component
            pBits++;
        }
    }
    fclose( file );
    free(filedata);
    DeleteDC( hdcImage );


    return DD_OK;
}
