//-----------------------------------------------------------------------
// File: TxtrLoad.cpp
//
// Desc: Support functions for loading textures, used by the texture
//       engine class.
//
//
// Copyright (C) 1997 Microsoft Corporation. All rights reserved
//-----------------------------------------------------------------------

#include <nvidia.h>
#include <d3dframework.h>


//-----------------------------------------------------------------------
// Function prototypes and static variable
//-----------------------------------------------------------------------
inline BOOL FileExists( TCHAR* n ) 
{ FILE* fp=_tfopen(n,TEXT("rb")); return fp?(0==fclose(fp)):FALSE; }

static BOOL LoadPPMHeader( FILE*, DWORD*, DWORD*, DWORD* );

static TCHAR* g_strRegValueName = TEXT("SDK");




//-----------------------------------------------------------------------
// Name: FindTextureFile()
// Desc: Looks for the specified file in the current directory, in the path
//       specified in the environment variables, or in the path specified in
//       the registry. After finding it, the function returns the full path.
//-----------------------------------------------------------------------
HRESULT FindTextureFile( TCHAR* strFilename, TCHAR* strTexturePath,
                        TCHAR* strFullPath )
{
  // First, check the current directory
  _tcscpy( strFullPath, strFilename );
  if( FileExists( strFullPath ) )
    return DD_OK;
  
  // Next, check to see if an environment variable specifies the path
  TCHAR* strPath = _tgetenv( TEXT("D3DPATH") );
  if( NULL != strPath )
  {
    _stprintf( strFullPath, TEXT("%s\\%s"), strPath, strFilename );
    if( FileExists( strFullPath ) )
      return DD_OK;
  }
  

  _stprintf( strFullPath, TEXT("..\\media\\%s"), strFilename );
  if( FileExists( strFullPath ) )
    return DD_OK;
  
  _stprintf( strFullPath, TEXT("..\\media\\lightmaps\\%s"), strFilename );
  if( FileExists( strFullPath ) )
    return DD_OK;
  
  
  _stprintf( strFullPath, TEXT("..\\..\\media\\%s"), strFilename );
  if( FileExists( strFullPath ) )
    return DD_OK;
  
  _stprintf( strFullPath, TEXT("..\\..\\media\\lightmaps\\%s"), strFilename );
  if( FileExists( strFullPath ) )
    return DD_OK;

  // Finally, check the system registry for a path
  HKEY key;
  LONG result = RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
    TEXT("Software\\NVIDIA"), 
    0, KEY_READ, &key );
  if( ERROR_SUCCESS == result )
  {
    TCHAR  strPath[512];
    DWORD type, size = 512;
    result = RegQueryValueEx( key, g_strRegValueName, NULL, &type, 
      (BYTE*)strPath, &size );
    
    RegCloseKey( key );
    
    if( ERROR_SUCCESS == result )
    {
      _stprintf( strFullPath, TEXT("%s\\Media\\%s"), strPath, 
        strFilename );
      if( FileExists( strFullPath ) )
        return DD_OK;
      
      _stprintf( strFullPath, TEXT("%s\\%s%s"), strPath, 
        strTexturePath, strFilename );
      if( FileExists( strFullPath ) )
        return DD_OK;

   _stprintf( strFullPath, TEXT("..\\media\\%s"), strFilename );
  if( FileExists( strFullPath ) )
    return DD_OK;
  
  _stprintf( strFullPath, TEXT("..\\media\\lightmaps\\%s"), strFilename );
  if( FileExists( strFullPath ) )
    return DD_OK;
  
  
  _stprintf( strFullPath, TEXT("..\\..\\media\\%s"), strFilename );
  if( FileExists( strFullPath ) )
    return DD_OK;
  
  _stprintf( strFullPath, TEXT("..\\..\\media\\lightmaps\\%s"), strFilename );
  if( FileExists( strFullPath ) )
    return DD_OK;
    }
  }
  
  error("Texture not found %s", strFilename);

  return DDERR_NOTFOUND;
}







//-----------------------------------------------------------------------
// Name: LoadBMPFile()
// Desc: Used as a callback to load textures, this function reads in a BMP
//       file and creates and returns a BITMAP handle for it.
//-----------------------------------------------------------------------
HRESULT LoadBMPFile( TCHAR* strFilename, HBITMAP* phbm )
{ 
    // Check params
    if( NULL==strFilename || NULL==phbm )
        return DDERR_INVALIDPARAMS;

    // Try to load the bitmap as a resource.
    (*phbm) = (HBITMAP)LoadImage( GetModuleHandle(NULL), strFilename, 
                              IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION ); 

    // If the bitmap wasn't a resource, try it as a file.
    if( NULL == (*phbm) ) 
        (*phbm) = (HBITMAP)LoadImage( NULL, strFilename, IMAGE_BITMAP, 0, 0, 
                                  LR_LOADFROMFILE|LR_CREATEDIBSECTION ); 

    return (*phbm) ? DD_OK : DDERR_NOTFOUND;
}




