#ifndef ALGEBRA_H
#define ALGEBRA_H
/***************************************************************/

#include <basic_types.h>
#include <iostream.h>
#include <stdlib.h>
#include <math.h>

#ifndef M_PI
#define M_PI          3.1415926535897932384626433832795028841971693993751
#endif
// this line defines a new type: pointer to a function which returns a
// Float and takes as argument a Floa


typedef Float (*V_FCT_PTR)(Float);

// min-max macros
//#define MIN(A,B) ((A) < (B) ? (A) : (B))
//#define MAX(A,B) ((A) > (B) ? (A) : (B))

// error handling macro
#define A_ERROR(E) { cerr << E; exit(101); }

class Vec2;
class Vec3;
class Vec4;
class Mat3;
class Mat4;

enum {X, Y, Z, W, U, V};		    // axes

enum {PA, PB, PC, PD};		    // planes
enum {RED, GREEN, BLUE};	    // colors
enum {KA, KD, KS, ES};		    // phong coefficients

/****************************************************************
*								*
*			    2D Vector				*
*								*
****************************************************************/

class Vec2
{
//protected:
public:

  union
  {
    Float n[2];
    struct
    {
      Float x,y;
    };
  };



public:

// Constructors

Vec2();
Vec2(const Float x, const Float y);
Vec2(const Float d);
Vec2(const Vec2& v);			// copy constructor
Vec2(const Vec3& v);			// cast v3 to v2
Vec2(const Vec3& v, int dropAxis);	// cast v3 to v2

// Assignment operators

Vec2& operator	= ( const Vec2& v );	// assignment of a Vec2
Vec2& operator += ( const Vec2& v );	// incrementation by a Vec2
Vec2& operator -= ( const Vec2& v );	// decrementation by a Vec2
Vec2& operator *= ( const Float d );	// multiplication by a constant
Vec2& operator /= ( const Float d );	// division by a constant
Float& operator [] ( int i); //const;		// indexing

// special functions

Float length() const;			// length of a Vec2
Float length2() const;			// squared length of a Vec2
Vec2& normalize();			// normalize a Vec2
Vec2& apply(V_FCT_PTR fct);		// apply a func. to each component

// friends

//friend Vec2 operator - (const Vec2& v);			    // -v1
//friend Vec2 operator + (const Vec2& a, const Vec2& b);	    // v1 + v2
//friend Vec2 operator - (const Vec2& a, const Vec2& b);	    // v1 - v2
//friend Vec2 operator * (const Vec2& a, const Float d);	    // v1 * 3.0
////friend Vec2 operator * (const Float d, const Vec2& a);	    // 3.0 * v1
//friend Vec2 operator * (const Mat3& a, const Vec2& v);	    // M . v
//friend Vec2 operator * (const Vec2& v, Mat3& a);	    // v . M
//friend Float operator * (const Vec2& a, const Vec2& b);    // dot product
//friend Vec2 operator / (const Vec2& a, const Float d);	    // v1 / 3.0
//friend Vec3 operator ^ (const Vec2& a, const Vec2& b);	    // cross product
//friend int operator == (const Vec2& a, const Vec2& b);	    // v1 == v2 ?
//friend int operator != (const Vec2& a, const Vec2& b);	    // v1 != v2 ?
//friend ostream& operator << (ostream& s, Vec2& v);	    // output to stream
//friend istream& operator >> (istream& s, Vec2& v);	    // input from strm.
//friend void swap(Vec2& a, Vec2& b);			    // swap v1 & v2
//friend Vec2 min(const Vec2& a, const Vec2& b);		    // min(v1, v2)
//friend Vec2 max(const Vec2& a, const Vec2& b);		    // max(v1, v2)
//friend Vec2 prod(const Vec2& a, const Vec2& b);		    // term by term *

// necessary //friend declarations

//friend class Vec3;
};

class q_type;
/****************************************************************
*								*
*			    3D Vector				*
*								*
****************************************************************/

class Vec3
{
  //friend q_type;
//protected:
public:

  union
  {
    Float n[3];
    struct
    {
      Float x,y,z;
    };
  };


public:

// Constructors

Vec3();
Vec3(const Float x, const Float y, const Float z);
Vec3(const Float d);
Vec3(const Vec3& v);			    // copy constructor
Vec3(const Vec2& v);			    // cast v2 to v3
Vec3(const Vec2& v, Float d);		    // cast v2 to v3
Vec3(const Vec4& v);			    // cast v4 to v3
Vec3(const Vec4& v, int dropAxis);	    // cast v4 to v3
Vec3(const Float f[3]);

#ifdef D3DVECTOR
Vec3( const D3DVECTOR& aVector ) : x( aVector.x ), y( aVector.y ), z( aVector.z ){;}
#endif

// Assignment operators

Vec3 & operator	= ( const Vec3& v );	    // assignment of a Vec3
Vec3 & operator += ( const Vec3& v );	    // incrementation by a Vec3
Vec3 & operator -= ( const Vec3& v );	    // decrementation by a Vec3
Vec3 & operator *= ( const Float d );	    // multiplication by a constant
Vec3 & operator /= ( const Float d );	    // division by a constant
Float& operator [] ( int i) { return n[i]; };		    // indexing

// special functions

Float length() const;			    // length of a Vec3
void tolerate();			    // length of a Vec3
Float length2() const;			    // squared length of a Vec3
Vec3& normalize();			    // normalize a Vec3
Vec3& apply(V_FCT_PTR fct);		    // apply a func. to each component

// //friends

friend Vec3 operator - (const Vec3& v);			    // -v1
friend Vec3 operator + (const Vec3& a, const Vec3& b);	    // v1 + v2
friend Vec3 operator - (const Vec3& a, const Vec3& b);	    // v1 - v2
friend Vec3 operator * (const Vec3& a, const Float d);	    // v1 * 3.0
friend Vec3 operator * (const Float d, const Vec3& a);	    // 3.0 * v1
friend Vec3 operator * (const Mat4& a, const Vec3& v);	    // M . v
friend Vec3 operator * (const Vec3& v, Mat4& a);	    // v . M
friend Float operator & (const Vec3& a, const Vec3& b);    // dot product
friend Vec3 operator * (const Vec3& a, const Vec3& b);    // Vec3 * Vec3
friend Vec3 operator / (const Vec3& a, const Float d);	    // v1 / 3.0
friend Vec3 operator ^ (const Vec3& a, const Vec3& b);	    // cross product
friend int operator == (const Vec3& a, const Vec3& b);	    // v1 == v2 ?
friend int operator != (const Vec3& a, const Vec3& b);	    // v1 != v2 ?
friend ostream& operator << (ostream& s, Vec3& v);	    // output to stream
friend istream& operator >> (istream& s, Vec3& v);	    // input from strm.
friend void swap(Vec3& a, Vec3& b);			    // swap v1 & v2
//friend Vec3 min(const Vec3& a, const Vec3& b);		    // min(v1, v2)
//friend Vec3 max(const Vec3& a, const Vec3& b);		    // max(v1, v2)
friend Vec3 prod(const Vec3& a, const Vec3& b);		    // term by term *
//    Vec3(Float f);
//    Vec3(Float _x, Float _y, Float _z);
//    Vec3(const Float f[3]);

    // =====================================
    // Access grants
    // =====================================

    const Float&operator[](int i) const;
//    Float&operator[](int i);

    // =====================================
    // Assignment operators
    // =====================================

//    Vec3& operator += (const Vec3& v);
//    Vec3& operator -= (const Vec3& v);
    Vec3& operator *= (const Vec3& v);
    Vec3& operator /= (const Vec3& v);
//    Vec3& operator *= (Float s);
//    Vec3& operator /= (Float s);

    // =====================================
    // Unary operators
    // =====================================

    friend Vec3 operator + (const Vec3& v);
    friend Vec3 operator - (const Vec3& v);


    // =====================================
    // Binary operators
    // =====================================

    // Addition and subtraction
        friend Vec3 operator + (const Vec3& v1, const Vec3& v2);
        friend Vec3 operator - (const Vec3& v1, const Vec3& v2);
    // Scalar multiplication and division
        friend Vec3 operator * (const Vec3& v, Float s);
        friend Vec3 operator * (Float s, const Vec3& v);
        friend Vec3 operator / (const Vec3& v, Float s);
    // Memberwise multiplication and division
        friend Vec3 operator * (const Vec3& v1, const Vec3& v2);
        friend Vec3 operator / (const Vec3& v1, const Vec3& v2);

    // Vector dominance
        friend int operator < (const Vec3& v1, const Vec3& v2);
        friend int operator <= (const Vec3& v1, const Vec3& v2);

    // Bitwise equality
        friend int operator == (const Vec3& v1, const Vec3& v2);

    // Length-related functions
        friend Float SquareMagnitude (const Vec3& v);
        friend Float Magnitude (const Vec3& v);

    // Returns vector with same direction and unit length
        friend Vec3 Normalize (const Vec3& v);

    // Return min/max component of the input vector
        friend Float Min (const Vec3& v);
        friend Float Max (const Vec3& v);

    // Return memberwise min/max of input vectors
        friend Vec3 Minimize (const Vec3& v1, const Vec3& v2);
        friend Vec3 Maximize (const Vec3& v1, const Vec3& v2);

    // Dot and cross product
        friend Float DotProduct (const Vec3& v1, const Vec3& v2);
        friend Vec3 CrossProduct (const Vec3& v1, const Vec3& v2);

#ifdef D3DVECTOR
	// conversions
	D3DVECTOR operator( void ) const
	{
		return D3DVECTOR( f[ 0 ], f[ 1 ], f[ 2 ] );
	}
#endif // D3DVECTOR

// necessary //friend declarations

//friend class Vec2;
//friend class Vec4;
//friend class Mat3;
//friend Vec2 operator * (const Mat3& a, const Vec2& v);	    // linear transform
//friend Mat3 operator * (Mat3& a, Mat3& b);		    // matrix 3 product
//friend Vec3 operator * (const Mat3& a, const Vec3& v);	    // linear transform
};



inline
Vec3::Vec3(Float f)
{
    x = y = z = f;
}

inline
Vec3::Vec3(Float _x, Float _y, Float _z)
{
    x = _x; y = _y; z = _z;
}

inline
Vec3::Vec3(const Float f[3])
{
    x = f[0]; y = f[1]; z = f[2];
}

// =====================================
// Access grants
// =====================================

inline const Float& Vec3::operator[](int i) const
{
  return (&x)[i];
}

//inline Float&Vec3::operator[](int i)
//{
//    return (&x)[i];
//}


// =====================================
// Assignment operators
// =====================================

inline Vec3&
Vec3::operator += (const Vec3& v)
{
   x += v.x;   y += v.y;   z += v.z;
   return *this;
}

inline Vec3&
Vec3::operator -= (const Vec3& v)
{
   x -= v.x;   y -= v.y;   z -= v.z;
   return *this;
}

inline Vec3&
Vec3::operator *= (const Vec3& v)
{
   x *= v.x;   y *= v.y;   z *= v.z;
   return *this;
}

inline Vec3&
Vec3::operator /= (const Vec3& v)
{
   x /= v.x;   y /= v.y;   z /= v.z;
   return *this;
}

inline Vec3&
Vec3::operator *= (Float s)
{
   x *= s;   y *= s;   z *= s;
   return *this;
}

inline Vec3&
Vec3::operator /= (Float s)
{
   x /= s;   y /= s;   z /= s;
   return *this;
}

inline Vec3
operator + (const Vec3& v)
{
   return v;
}

inline Vec3
operator - (const Vec3& v)
{
   return Vec3(-v.x, -v.y, -v.z);
}

inline Vec3
operator + (const Vec3& v1, const Vec3& v2)
{
   return Vec3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z);
}

inline Vec3
operator - (const Vec3& v1, const Vec3& v2)
{
   return Vec3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z);
}

inline Vec3
operator * (const Vec3& v1, const Vec3& v2)
{
   return Vec3(v1.x*v2.x, v1.y*v2.y, v1.z*v2.z);
}

inline Vec3
operator / (const Vec3& v1, const Vec3& v2)
{
   return Vec3(v1.x/v2.x, v1.y/v2.y, v1.z/v2.z);
}

inline int
operator < (const Vec3& v1, const Vec3& v2)
{
   return v1[0] < v2[0] && v1[1] < v2[1] && v1[2] < v2[2];
}

inline int
operator <= (const Vec3& v1, const Vec3& v2)
{
   return v1[0] <= v2[0] && v1[1] <= v2[1] && v1[2] <= v2[2];
}

inline Vec3
operator * (const Vec3& v, Float s)
{
   return Vec3(s*v.x, s*v.y, s*v.z);
}

inline Vec3
operator * (Float s, const Vec3& v)
{
   return Vec3(s*v.x, s*v.y, s*v.z);
}

inline Vec3
operator / (const Vec3& v, Float s)
{
   return Vec3(v.x/s, v.y/s, v.z/s);
}

inline int
operator == (const Vec3& v1, const Vec3& v2)
{
   return v1.x==v2.x && v1.y==v2.y && v1.z == v2.z;
}

inline Float
Magnitude (const Vec3& v)
{
   return (Float) sqrt(SquareMagnitude(v));
}

inline Float
SquareMagnitude (const Vec3& v)
{
   return v.x*v.x + v.y*v.y + v.z*v.z;
}

inline Vec3
Normalize (const Vec3& v)
{
   return v / Magnitude(v);
}

inline Float
Min (const Vec3& v)
{
   Float ret = v.x;
   if (v.y < ret) ret = v.y;
   if (v.z < ret) ret = v.z;
   return ret;
}

inline Float
Max (const Vec3& v)
{
   Float ret = v.x;
   if (ret < v.y) ret = v.y;
   if (ret < v.z) ret = v.z;
   return ret;
}

inline Vec3
Minimize (const Vec3& v1, const Vec3& v2)
{
   return Vec3( v1[0] < v2[0] ? v1[0] : v2[0],
                   v1[1] < v2[1] ? v1[1] : v2[1],
                   v1[2] < v2[2] ? v1[2] : v2[2]);
}

inline Vec3
Maximize (const Vec3& v1, const Vec3& v2)
{
   return Vec3( v1[0] > v2[0] ? v1[0] : v2[0],
                   v1[1] > v2[1] ? v1[1] : v2[1],
                   v1[2] > v2[2] ? v1[2] : v2[2]);
}

inline Float
DotProduct (const Vec3& v1, const Vec3& v2)
{
   return v1.x*v2.x + v1.y * v2.y + v1.z*v2.z;
}



inline Vec3
CrossProduct (const Vec3& p, const Vec3& q)
{
	Vec3 cp;

  cp[X] = p[Y] * q[Z] - p[Z] * q[Y];
  cp[Y] = p[Z] * q[X] - p[X] * q[Z];
  cp[Z] = p[X] * q[Y] - p[Y] * q[X];
  
	return cp;
}


/*inline Vec3
CrossProduct (const Vec3& v1, const Vec3& v2)
{
	Vec3 result;

	result[0] = v1[1] * v2[2] - v1[2] * v2[1];
	result[1] = v1[2] * v2[0] - v1[0] * v2[2];
	result[2] = v1[0] * v2[1] - v1[1] * v2[0];

	return result;
}*


/****************************************************************
*								*
*			    4D Vector				*
*								*
****************************************************************/

class Vec4
{
public:
//protected:

  union
  {
    Float n[4];
    struct
    {
      Float x,y,z,w;
    };
  };

public:

// Constructors

Vec4();
Vec4(const Float x, const Float y, const Float z, const Float w);
Vec4(const Float d);
Vec4(const Vec4& v);			    // copy constructor
Vec4(const Vec3& v);			    // cast Vec3 to Vec4
Vec4(const Vec3& v, const Float d);	    // cast Vec3 to Vec4

// Assignment operators

Vec4& operator	= ( const Vec4& v );	    // assignment of a Vec4
Vec4& operator += ( const Vec4& v );	    // incrementation by a Vec4
Vec4& operator -= ( const Vec4& v );	    // decrementation by a Vec4
Vec4& operator *= ( const Float d );	    // multiplication by a constant
Vec4& operator /= ( const Float d );	    // division by a constant
Float& operator [] ( int i); //const;		    // indexing

// special functions

Float length() const;			    // length of a Vec4
Float length2() const;			    // squared length of a Vec4
Vec4& normalize();			    // normalize a Vec4
inline void Homogenize( void );
Vec4& apply(V_FCT_PTR fct);		    // apply a func. to each component

// //friends

friend Vec4 operator - (const Vec4& v);			    // -v1
friend Vec4 operator + (const Vec4& a, const Vec4& b);	    // v1 + v2
friend Vec4 operator - (const Vec4& a, const Vec4& b);	    // v1 - v2
friend Vec4 operator * (const Vec4& a, const Float d);	    // v1 * 3.0
friend Vec4 operator * (const Float d, const Vec4& a);	    // 3.0 * v1
friend Vec4 operator * (const Mat4& a, const Vec4& v);	    // M . v
friend Vec4 operator * (const Vec4& v, Mat4& a);	    // v . M
friend Float operator * (const Vec4& a, const Vec4& b);    // dot product
friend Vec4 operator / (const Vec4& a, const Float d);	    // v1 / 3.0
friend int operator == (const Vec4& a, const Vec4& b);	    // v1 == v2 ?
friend int operator != (const Vec4& a, const Vec4& b);	    // v1 != v2 ?
friend ostream& operator << (ostream& s, Vec4& v);	    // output to stream
friend istream& operator >> (istream& s, Vec4& v);	    // input from strm.
friend void swap(Vec4& a, Vec4& b);			    // swap v1 & v2
//friend Vec4 min(const Vec4& a, const Vec4& b);		    // min(v1, v2)
//friend Vec4 max(const Vec4& a, const Vec4& b);		    // max(v1, v2)
friend Vec4 prod(const Vec4& a, const Vec4& b);		    // term by term *

// necessary //friend declarations

//friend class Vec3;
//friend class Mat4;
//friend Vec3 operator * (const Mat4& a, const Vec3& v);	    // linear transform
//friend Mat4 operator * (Mat4& a, Mat4& b);		    // matrix 4 product
};

inline void Vec4::Homogenize( void )
{
  n[0] /= n[3];
  n[1] /= n[3];
  n[2] /= n[3];
}





/****************************************************************
*								*
*			   3x3 Matrix				*
*								*
****************************************************************/

class Mat3
{
public:
  Float m[3][3];
protected:

 //Vec3 v[3];

public:

// Constructors

Mat3();
Mat3(const Vec3& v0, const Vec3& v1, const Vec3& v2);
Mat3(const Float m00, const Float m01, const Float m02,
     const Float m10, const Float m11, const Float m12,
     const Float m20, const Float m21, const Float m22);
Mat3(const Float d);
Mat3(const Mat3& m);

// Assignment operators

Mat3& operator	= ( const Mat3& m );	    // assignment of a Mat3
Mat3& operator += ( const Mat3& m );	    // incrementation by a Mat3
Mat3& operator -= ( const Mat3& m );	    // decrementation by a Mat3
Mat3& operator *= ( const Float d );	    // multiplication by a constant
Mat3& operator /= ( const Float d );	    // division by a constant
//Vec3& operator [] ( const int i);		    // indexing
//Float & operator () ( const int i, const int j);		    // indexing

// special functions

Mat3 transpose();			    // transpose
Mat3 inverse();				    // inverse
Mat3& apply(V_FCT_PTR fct);		    // apply a func. to each element

// //friends

friend Mat3 operator - (const Mat3& a);			    // -m1
friend Mat3 operator + (const Mat3& a, const Mat3& b);	    // m1 + m2
friend Mat3 operator - (const Mat3& a, const Mat3& b);	    // m1 - m2
friend Mat3 operator * (Mat3& a, Mat3& b);		    // m1 * m2
friend Mat3 operator * (const Mat3& a, const Float d);	    // m1 * 3.0
friend Mat3 operator * (const Float d, const Mat3& a);	    // 3.0 * m1
friend Mat3 operator / (const Mat3& a, const Float d);	    // m1 / 3.0
friend int operator == (const Mat3& a, const Mat3& b);	    // m1 == m2 ?
friend int operator != (const Mat3& a, const Mat3& b);	    // m1 != m2 ?
friend ostream& operator << (ostream& s, Mat3& m);	    // output to stream
friend istream& operator >> (istream& s, Mat3& m);	    // input from strm.
friend void swap(Mat3& a, Mat3& b);			    // swap m1 & m2

// necessary //friend declarations

friend Vec3 operator * (const Mat3& a, const Vec3& v);	    // linear transform
//friend Vec2 operator * (const Mat3& a, const Vec2& v);	    // linear transform
};

/****************************************************************
*								*
*			   4x4 Matrix				*
*								*
****************************************************************/

class Mat4
{
public:
  Float m[4][4];

// Constructors

Mat4();
Mat4(Float a0, Float b0, Float c0, Float d0,
     Float a1, Float b1, Float c1, Float d1,
     Float a2, Float b2, Float c2, Float d2,
     Float a3, Float b3, Float c3, Float d3);

Mat4(const Vec4& v0, const Vec4& v1, const Vec4& v2, const Vec4& v3);
Mat4(const Float d);
Mat4(const Mat4& m);

// Assignment operators

Mat4& operator	= ( const Mat4& m );	    // assignment of a Mat4
Mat4& operator	= ( const Mat3& m );	    // assignment of a Mat4
Mat4& operator += ( const Mat4& m );	    // incrementation by a Mat4
Mat4& operator -= ( const Mat4& m );	    // decrementation by a Mat4
Mat4& operator *= ( const Float d );	    // multiplication by a constant
Mat4& operator /= ( const Float d );	    // division by a constant
Float & operator () ( const int i, const int j);// const;		    // indexing
//Vec4& operator [] ( int i);		    // indexing

// special functions

Mat4 transpose();			    // transpose
Mat4 inverse();				    // inverse
Mat4 identity();				    // identity
Mat4& apply(V_FCT_PTR fct);		    // apply a func. to each element

// //friends

friend Mat4 operator - (const Mat4& a);			    // -m1
friend Mat4 operator + (const Mat4& a, const Mat4& b);	    // m1 + m2
friend Mat4 operator - (const Mat4& a, const Mat4& b);	    // m1 - m2
friend Mat4 operator * (Mat4& a, Mat4& b);		    // m1 * m2
friend Mat4 operator * (const Mat4& a, const Float d);	    // m1 * 4.0
friend Mat4 operator * (const Float d, const Mat4& a);	    // 4.0 * m1
friend Mat4 operator / (const Mat4& a, const Float d);	    // m1 / 3.0
friend int operator == (const Mat4& a, const Mat4& b);	    // m1 == m2 ?
friend int operator != (const Mat4& a, const Mat4& b);	    // m1 != m2 ?
friend ostream& operator << (ostream& s, Mat4& m);	    // output to stream
friend istream& operator >> (istream& s, Mat4& m);	    // input from strm.
friend void swap(Mat4& a, Mat4& b);			    // swap m1 & m2

// necessary //friend declarations

//friend Vec4 operator * (const Mat4& a, const Vec4& v);	    // linear transform
//friend Vec3 operator * (const Mat4& a, const Vec3& v);	    // linear transform
};

/****************************************************************
*								*
*	       2D functions and 3D functions			*
*								*
****************************************************************/

Mat3 identity2D();					    // identity 2D
Mat3 translation2D(Vec2& v);				    // translation 2D
Mat3 rotation2D(Vec2& Center, const Float angleDeg);	    // rotation 2D
Mat3 scaling2D(Vec2& scaleVector);			    // scaling 2D
Mat4 identity3D();					    // identity 3D
Mat4 translation3D(Vec3& v);				    // translation 3D
Mat4 rotation3D(Vec3& Axis, const Float angleDeg);	    // rotation 3D
Mat4 scaling3D(Vec3& scaleVector);			    // scaling 3D
Mat4 perspective3D(const Float d);			    // perspective 3D




// QUAT

class q_type
{

protected:
 Float q[4];

public:

// Constructors

q_type();
q_type(const Float x, const Float y, const Float z, const Float w);
q_type(const Float d);
q_type(const q_type& v);			    // copy constructor
q_type(const Vec3& v);			    // cast Vec3 to q_type
q_type(const Vec3& v, const Float d);	    // cast Vec3 to q_type

// Assignment operators

q_type & operator	= ( const q_type& v );	    // assignment of a q_type
q_type & operator += ( const q_type& v );	    // incrementation by a q_type
q_type & operator -= ( const q_type& v );	    // decrementation by a q_type
q_type & operator *= ( const Float d );	    // multiplication by a constant
q_type & operator /= ( const Float d );	    // division by a constant
Float & operator [] ( int i);// const;		    // indexing

// special functions

Float length() const;			    // length of a q_type
Float length2() const;			    // squared length of a q_type
q_type& normalize();			    // normalize a q_type
q_type& apply(V_FCT_PTR fct);		    // apply a func. to each component

// //friends

friend q_type operator - (const q_type& v);			    // -v1
friend q_type operator + (const q_type& a, const q_type& b);	    // v1 + v2
friend q_type operator - (const q_type& a, const q_type& b);	    // v1 - v2
friend q_type operator * (const q_type& a, const Float d);	    // v1 * 3.0
friend q_type operator * (const Float d, const q_type& a);	    // 3.0 * v1
friend q_type operator * (const Mat4& a, const q_type& v);	    // M . v
friend q_type operator * (const q_type& v, Mat4& a);	    // v . M
friend q_type operator * (const q_type& a, const q_type& b);    // mult quat
friend q_type operator * (const Vec3& a, const q_type& b);    // mult vec *quat
friend q_type operator * (const q_type& a, const Vec3& b);    // mult quat*vec
friend q_type operator / (const q_type& a, const Float d);	    // v1 / 3.0
friend int operator == (const q_type& a, const q_type& b);	    // v1 == v2 ?
friend int operator != (const q_type& a, const q_type& b);	    // v1 != v2 ?
friend ostream& operator << (ostream& s, q_type& v);	    // output to stream
friend istream& operator >> (istream& s, q_type& v);	    // input from strm.
//friend void swap(q_type& a, q_type& b);			    // swap v1 & v2
//friend q_type min(const q_type& a, const q_type& b);		    // min(v1, v2)
//friend q_type max(const q_type& a, const q_type& b);		    // max(v1, v2)
//friend q_type prod(const q_type& a, const q_type& b);		    // term by term *

// necessary //friend declarations

//friend class Vec3;
//friend class Mat4;
//friend Vec3 operator * (const Mat4& a, const Vec3& v);	    // linear transform
//friend Mat4 operator * (Mat4& a, Mat4& b);		    // matrix 4 product
};


typedef class Vec3 Triple;

//typedef class Mat4 q_matrix_type;
//typedef class Vec3 col_Vect3;
//typedef class Mat3 col_Mat3;
//typedef class Mat4 col_Mat4;
//typedef class Vec4 col_Vect4;
//typedef class Vec4 Vec4;
//typedef class Vec3 Vec3;
//typedef class Mat3 Mat3;
//typedef class Mat4 Mat4;


typedef void (*dydt_func)(void * tope, Float t, Float y[], Float * ydot);





#include <algebra.def>


typedef struct Plane
{
  union
  {
    struct 
    {
      Vec3 vec;
      Float D;
    };
    struct 
    {
      Float x,y,z,d;
    };
  };
} Plane;




#endif

