#include <nvidia.h>
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

#include "tga.h"
typedef unsigned char Byte;
//typedef int Int;

#define VERBOSE 0

static Byte id_length;          /* 0 */
static Byte color_map_type;     /* 1 */
static Byte image_type;         /* 2 */
static Int cm_index;            /* 3 4 */
static Int cm_length;           /* 5 6 */
static Byte cm_entry_size;      /* 7 */
static Int image_x_org;         /* 8 9 */
static Int image_y_org;         /* 10 11 */
static Int image_width;         /* 12 13 */
static Int image_height;        /* 14 15 */
static Byte image_depth;        /* 16 */
static Byte image_desc;         /* 17 */

static Byte image_id[256];

static Byte color_palette[256 * 3];

static FILE *fpin;

#define BPP 8
#define LR 0x10
#define BT 0x20

#define bread(fp) (Byte)fgetc(fp)

int
wread(FILE * fp)
{
  Byte h, l;

  l = (Byte) fgetc(fp);
  h = (Byte) fgetc(fp);
  return (h << 8) | l;
}

/*

   Use this to read the whole tga file, allocate memory and return a pointer to the memory,

  reads 24/32 tga -> 24 or 32 bpp
  does not support 8 bpp palette

  flags
  swap: 1 = swaps rgb position
  pad = pads 24 bpp to 32 bpp (sets alpha to 0xFF)



*/



unsigned char *LoadTGAHeader (FILE * fpin, DWORD * width, DWORD * height, 
                              int swap, int pad)
{
  int i, rle;
  int a, r, g, b;
  int right, top;
  unsigned char *data;
  int size;
  unsigned char *result;


  /*if ((fpin = fopen(filename, "rb")) == NULL)
    {
      printf("Cannot open file '%s'\n", filename);
      exit(111);
    }
*/
  //index = 2;

  id_length = bread(fpin);
  color_map_type = bread(fpin);
  image_type = bread(fpin);

  if (!(image_type == 2 || image_type == 10))
    {
      printf("Only type 2 or 10 supported\n");
      exit(112);
    }
  cm_index = wread(fpin);
  cm_length = wread(fpin);

  cm_entry_size = bread(fpin);

  image_x_org = wread(fpin);
  image_y_org = wread(fpin);
  image_width = wread(fpin);
  image_height = wread(fpin);

  image_depth = bread(fpin);
  image_desc = bread(fpin);
#if VERBOSE

  printf("id_length %d\n", id_length);
  printf("color_map_type %d\n", color_map_type);
  printf("image_type %d\n", image_type);
  printf("index %d\n", cm_index);
  printf("length %d\n", cm_length);
  printf("entry_size %d\n", cm_entry_size);
  printf("x_org %d\n", image_x_org);
  printf("y_org %d\n", image_y_org);
  printf("width %d\n", image_width);
  printf("height %d\n", image_height);
  printf("depth %d\n", image_depth);
  printf("desc %02X\n", image_desc);
#endif
  *width = image_width;
  *height = image_height;



//unsigned char *
//read_tga(char *filename, unsigned char *result, int *row, int *col, int pad, int swap)
//{



  //printf("image file specs %d x %d\n", image_width, image_height);

  result = (Byte *) malloc(image_width * image_height * 4);
  if (result == 0)
    return NULL;
 

  right = image_desc & LR;
  top = image_desc & BT;

  fread((char *) image_id, id_length, 1, fpin);
  fread((char *) color_palette, cm_length, cm_entry_size >> 3, fpin);

  data = result;

  size = (int) image_height *(int) image_width;

  if (image_type == 2)
  {
    if (image_depth == 24)
    {
      for (i = 0; i < size; i++)
      {
        if (swap)
        {
          b = fgetc(fpin);
          g = fgetc(fpin);
          r = fgetc(fpin);
          *data++ = r;
          *data++ = g;
          *data++ = b;
        }
        else
        {
          *data++ = fgetc(fpin);
          *data++ = fgetc(fpin);
          *data++ = fgetc(fpin);
        }
        
        if (pad)
          *data++ = 0xFF;
      }
    }
    else
    {
      // 32 bit
      for (i = 0; i < size; i++)
      {
        if (swap)
        {
          b = fgetc(fpin);
          g = fgetc(fpin);
          r = fgetc(fpin);
          a = fgetc(fpin);
          *data++ = r;
          *data++ = g;
          *data++ = b;
          *data++ = a;
        }
        else
        {
          *data++ = fgetc(fpin);
          *data++ = fgetc(fpin);
          *data++ = fgetc(fpin);
          *data++ = fgetc(fpin);
        }
      }
    }
  }
  else                          //RLE
  {
    if (image_depth == 24)
    {
      i = 0;
      while (i < size)
      {
        
        rle = fgetc(fpin);
        if (rle < 0x80)
        {
          rle++;
          while (rle)
          {
            if (swap)
            {
              b = fgetc(fpin);
              g = fgetc(fpin);
              r = fgetc(fpin);
              *data++ = r;
              *data++ = g;
              *data++ = b;
            }
            else
            {
              *data++ = fgetc(fpin);
              *data++ = fgetc(fpin);
              *data++ = fgetc(fpin);
            }
            if (pad)
              *data++ = 0xFF;
            rle--;
            i++;
            if (i > size)
            {
              printf("RLE overflow\n");
              exit(113);
            }
          }
        }
        else
        {
          rle -= 127;
          b = fgetc(fpin);
          g = fgetc(fpin);
          r = fgetc(fpin);
          
          while (rle)
          {
            if (swap)
            {
              *data++ = r;
              *data++ = g;
              *data++ = b;
            }
            else
            {
              *data++ = b;
              *data++ = g;
              *data++ = r;
            }
            if (pad)
              *data++ = 0xFF;
            i++;
            if (i > size)
            {
              printf("RLE overflow\n");
              exit(115);
            }
            rle--;
          }
        }
        
      }
    }
    else                      // 32bit RLE
    {
      i = 0;
      while (i < size)
      {
        
        rle = fgetc(fpin);
        if (rle < 0x80)
        {
          rle++;
          while (rle)
          {
            if (swap)
            {
              b = fgetc(fpin);
              g = fgetc(fpin);
              r = fgetc(fpin);
              a = fgetc(fpin);
              *data++ = r;
              *data++ = g;
              *data++ = b;
              *data++ = a;
            }
            else
            {
              
              *data++ = fgetc(fpin);
              *data++ = fgetc(fpin);
              *data++ = fgetc(fpin);
              *data++ = fgetc(fpin);
            }
            rle--;
            i++;
            if (i > size)
            {
              printf("RLE overflow\n");
              exit(116);
            }
            
          }
        }
        else
        {
          rle -= 127;
          b = fgetc(fpin);
          g = fgetc(fpin);
          r = fgetc(fpin);
          a = fgetc(fpin);
          
          while (rle)
          {
            if (swap)
            {
              *data++ = r;
              *data++ = g;
              *data++ = b;
              *data++ = a;
            }
            else
            {
              *data++ = b;
              *data++ = g;
              *data++ = r;
              *data++ = a;
            }
            rle--;
            i++;
            if (i > size)
            {
              printf("RLE overflow\n");
              exit(117);
            }
          }
        }
        
      }
    }
  }
  fclose(fpin);
  
  return result;
}
