#ifndef BASIC_TYPES_H
#define BASIC_TYPES_H



#include <std.h>
#include "pragma.h"


enum {NO, YES};


#ifndef FALSE
#define FALSE               0
#endif

#ifndef TRUE
#define TRUE                1
#endif

enum {FAIL, PASS};
enum {OFF, ON};
enum VelocityType {NEGATIVE, RESTING, POSITIVE};

typedef double Float;

typedef float Float32;
typedef double Float64;
//typedef int Int;
//typedef unsigned int uInt;
typedef long Int; 
typedef unsigned long uInt;

typedef char     Char;
typedef unsigned char     Byte;
typedef unsigned short    uShort;

typedef unsigned __int64  uLong; 
typedef __int64  Long; 


typedef short    Short;



enum {VERTEX=1,EDGE, FACE};

#include <algebra.h>




#define BVERTEX1 1
#define BEDGE1 2
#define BFACE1 4
#define BVERTEX2 (1<<4)
#define BEDGE2 (2<<4)
#define BFACE2 (4<<4)


#define featTag(featurePtr) (*((int *) (featurePtr)))
#define SWAP(x,y,type) {type tmp; tmp = x; x = y; y = tmp;}


#define DEG2RAD      0.01745329251994
#define RAD2DEG     57.29577951308232

#define deg2rad(x)   ((x) * DEG2RAD)
#define rad2deg(x)   ((x) * RAD2DEG) 

#define ifnot(x)   if (!(x))
#define until(x) while(!(x))
#define ever          (;;)
#define wait        do {}
#define nothing     {}

#define EPSILON 1e-12
#define EPS 1e-12

#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))



#define SQR(x) ((x)*(x))
#define CUBE(x) ((x)*(x)*(x))

#ifndef PI
#define PI          3.1415926535897932384626433832795028841971693993751
#endif
#ifdef M_PI
#undef M_PI
#endif
#define M_PI          3.1415926535897932384626433832795028841971693993751
#define SQRT_2  1.4142135623730950488016887242096980785696718753769
#define PI2        (PI/2.0)

typedef enum
{
	RENDER_WIRE,
	RENDER_SOLID
} RenderTypes;


typedef enum
{
	OBJECT_SPHERE,
	OBJECT_PLANE 
} ObjectTypes;

/*#define dcos(x)  cos((x) * DEG2RAD)
#define dsin(x)  sin((x) * DEG2RAD)
#define mil_dcos(x)  cos((90-(x)) * DEG2RAD)
#define mil_dsin(x)  sin((90-(x)) * DEG2RAD)
#define dtan(x)  tan((x) * DEG2RAD)
*/



//const Int X = 0;
//const Int Y = 1;
//const Int Z = 2;
//const Int T = 3;
//const Int W = 3;






#define MAX_FLOAT (1e+300)


typedef Float64 LatLon;
/* FIXED prexision*/
typedef Int Fix;
typedef uInt uFix;


typedef struct Short2D
{
  Short x,y;
} Short2D;

typedef struct uShort2D
{
  uShort x,y;
} uShort2D;

typedef struct Srect
{
  Short x1, y1, x2, y2;
} Srect;


typedef struct Int2D
{
  Int x, y;
} Int2D;          


typedef struct Int3D
{
    Int x, y, z;
} Int3D;          


typedef struct Irect
{
    Int x1, y1, x2, y2;
} Irect;


typedef struct Irect3D
{
  Int x1, y1, x2, y2, z1, z2;
} Irect3D;


typedef struct Frect
{
  Float x1, y1, x2, y2;
} Frect;

typedef struct Frect2D
{
  Float x1, y1, x2, y2;
} Frect2D;

typedef struct Frect3D
{
  Float x1, y1, x2, y2, z1, z2;
} Frect3D;


typedef struct Float2D
{
  Float x, y;
} Float2D;

typedef struct Float3D
{
  Float x, y, z;
} Float3D;

//typedef struct Double3D
//{
//  Double x, y, z;
//} Double3D;



//typedef struct RGB
//{
//  Byte r,g,b;
//} RGB;



typedef struct FloatUV
{
  Float u, v;
} FloatUV;


typedef struct IntUV
{
  Int u, v;
} IntUV;



typedef struct LLE
{
  Float lat, lon, elev;
} LLE;



typedef union UPointer
{
  Char * c;  /* type 1 */
  Byte * b; /* type 2 */
  
  Short * s;  /* type 3 */
  uShort * w; /* type 4 */
  
  Int * i;    /* type 5 */
  uInt * l;   /* type 6 */
  
  Float32 * f;  /* type 7 */
  void ** p;  /* type 8 */
  Float64 * d;  /* type 9 */
  void * v;  /* type 10 */
} UPointer;



typedef Float Vector4D[4];
typedef Float Matrix4x4[4][4];



typedef struct Irect2D
{
  Int x1, y1, x2, y2;
} Irect2D;



/* FIXED  */


typedef struct Fix2D
{
  Fix x, y;
} Fix2D;




typedef struct Locator
{
  Float x,y;
  Float2D abs;
  Float2D rel;
  Int b1, b2, b3;

} Locator;

typedef struct Vertex2D
{
  Float x,y;
  Int region;
  Int type;
} Vertex2D;

typedef struct Instance2D
{
  Vertex2D * vertex;
} Instance2D;

typedef struct InstancePointer2D
{
  Int n;
  Instance2D ** ilist;
} InstancePointer2D;

typedef struct Vheader2D
{
  InstancePointer2D ip;   /* pointer to list of vertices */
  InstancePointer2D clipped_ip;
} Vheader2D;



#endif