//////////////////////////////////////////////////////////////////////
// MyLandscape.cpp: implementation of the MyLandscape class.
//
//		Quick and dirty terrain generation (Random not fractal or anything!)
//
// Copyright (C) 1999 NVIDIA Corporation. All rights reserved
//////////////////////////////////////////////////////////////////////

#include "MyLandscape.h"
#include <d3dframework.h>
extern CD3DFramework* g_pFramework;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////



//*************************************************************************
// Function name	: myRand10
// Description	    : Generate a random floating point number in range [0..10]
// Return type		: float 
//*************************************************************************
float myRand10(){
	return ((float)rand()/(float)RAND_MAX)*7.0f;
}



MyLandscape::MyLandscape(int _x,int _z)
{
	int i=0,j=0;
	int CurrentTri=0;

	float xScale = 300.0/(float)_x;
	float zScale = 300.0/(float)_z;
	x = _x;
	z = _z;

	pVertices=new D3DVERTEX[(x+1)*(z+1)];
	pTriangles=new TRI[2*x*z];
	int CurrentVert=0;
	//Fill vertex grid with vertices of random height
	//scales and reposition the landscape to make it easily visiible
	//also project a texture onto landscape
	for(j=0;j<z+1;j++)
	{
		for (i=0;i<x+1;i++)
		{
			CurrentVert=i+j*(x+1);
			pVertices[CurrentVert].x=i*xScale-150.0f;
			pVertices[CurrentVert].y=myRand10();
			pVertices[CurrentVert].z=-j*zScale;
			pVertices[CurrentVert].nx=0.0f;
			pVertices[CurrentVert].ny=1.0f;
			pVertices[CurrentVert].nz=0.0f;
			pVertices[CurrentVert].tu=(float)i/(float)(x);
			pVertices[CurrentVert].tv=(float)j/(float)(z);

		}
	}
	
	//generate face indices for landscape
	for(j=0;j<z;j++)
	{
		for (i=0;i<x;i++)
		{
			CurrentTri=2*(i+j*x);

			pTriangles[CurrentTri].v1=   i   + j*(x+1);
			pTriangles[CurrentTri].v2= (i+1) + j*(x+1);
			pTriangles[CurrentTri].v3=   i   + (j+1)*(x+1);

			CurrentTri++;
			
			pTriangles[CurrentTri].v1= (i+1) + j*(x+1);
			pTriangles[CurrentTri].v2= (i+1) + (j+1)*(x+1);
			pTriangles[CurrentTri].v3=   i   + (j+1)*(x+1);
		}
	}
	
	//Create a vertex buffer to hold untransformed object
	D3DVERTEXBUFFERDESC vbDesc;
	LPD3DVERTEX pvbVerts=NULL;


	memset( &vbDesc,0, sizeof(D3DVERTEXBUFFERDESC) );
	vbDesc.dwSize        = sizeof(D3DVERTEXBUFFERDESC);
	vbDesc.dwCaps        = 0L;
	vbDesc.dwFVF         = D3DFVF_VERTEX ;
	vbDesc.dwNumVertices = (x+1)*(z+1);

	g_pFramework->GetDirect3D()->CreateVertexBuffer(&vbDesc,&pVBObject,0,NULL);
	
	
	//fill it with the correct verts
	pVBObject->Lock( DDLOCK_WRITEONLY|DDLOCK_SURFACEMEMORYPTR |DDLOCK_NOSYSLOCK,(LPVOID *)&pvbVerts,NULL);

	for (int v=0;v<(x+1)*(z+1);v++){
		pvbVerts[v]=pVertices[v];
	}
	pVBObject->Unlock();
	
	//Create a vertex buffer to hold transformed object
	vbDesc.dwFVF		= D3DFVF_TLVERTEX;
	g_pFramework->GetDirect3D()->CreateVertexBuffer(&vbDesc,&pVBScreenSpace,0,NULL);
}

MyLandscape::~MyLandscape()
{
	x=z=0;
	
	delete [] pTriangles;pTriangles=NULL;
	delete [] pVertices;pVertices=NULL;
	if (pVBObject)
		pVBObject->Release();
	if (pVBScreenSpace)
		pVBScreenSpace->Release();

}


//*************************************************************************
// Function name	: MyLandscape::Render
// Description	    : Render the landscape. Uses process vertices to allow
//					: the explosion pass to read back the screenspace verts
//					  if required
// Return type		: void 
//*************************************************************************
void MyLandscape::Render(LPDIRECT3DDEVICE3 pDev)
{

	pVBScreenSpace->ProcessVertices(D3DVOP_CLIP|D3DVOP_TRANSFORM|D3DVOP_EXTENTS|D3DVOP_LIGHT,0,(x+1)*(z+1),pVBObject,0,pDev,0);

	pDev->DrawIndexedPrimitiveVB(D3DPT_TRIANGLELIST,pVBScreenSpace,(WORD *)pTriangles,3*2*x*z,D3DDP_DONOTUPDATEEXTENTS);
}
