//-----------------------------------------------------------------------------
// File: Sample.cpp
//
// Desc: Example code showing how to do volumetric explosions
//       Note: This code uses the D3D Framework helper library.
//
//
// Copyright (c) 1999 NVIDIA Corporation. All rights reserved.
//-----------------------------------------------------------------------------

#include <nvidia.h>
#include <d3dframework.h>

#include "mylandscape.h"
#include "Explosion.h"

extern CD3DFramework* g_pFramework;
//-----------------------------------------------------------------------------
// Declare the application globals for use in WinMain.cpp
//-----------------------------------------------------------------------------
TCHAR* g_strAppTitle       = TEXT( "Nvidia Volumetric Explosion" );
BOOL   g_bAppUseZBuffer    = TRUE;    // Create/use a z-buffering
BOOL   g_bAppUseBackBuffer = TRUE;    // Create/use a back buffer
BOOL   g_bAppUseStencil    = FALSE;    // Create/use a stencil buffer
BOOL   g_bAppUseRefRast    = TRUE;   // Enumerate the reference rasterizer
DWORD  g_FilterMode        = D3DTFG_LINEAR;
float g_fZBias = 8.0f/65536.0f;
BOOL g_bRenderBB = TRUE;

BOOL g_bTwoExplosions = FALSE;
TCHAR	*g_textureFile0[] = {"ground.ppm",
"exp01.ppm",
"exp02.ppm",
"exp03.ppm",
"exp04.ppm",
"exp05.ppm",
"exp06.ppm",
"exp07.ppm",
"exp08.ppm",
"exp09.ppm",
"exp10.ppm",
"exp11.ppm",
"exp12.ppm",
"exp13.ppm",
"exp14.ppm",
"exp15.ppm",
"exp16.ppm"
};


//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------

LPDIRECT3DMATERIAL3 g_pmtrlBackgroundMtrl = NULL;
LPDIRECT3DMATERIAL3 g_pmtrlObjectMtrl     = NULL;
LPDIRECT3DLIGHT     g_pLight              = NULL;
D3DMATRIX			g_matView ;


MyLandscape* g_pLandscape;
Explosion* g_pExplosion;
Explosion* g_pExplosion2;

EXPLOSIONTYPE g_ExpType=EXP_VOLUMETRICWORLDSPACE; 

BOOL bWireframe=FALSE;
BOOL bToggleWireframe=FALSE;


BOOL bTexture=TRUE;

//-----------------------------------------------------------------------------
// Function prototypes and global (or static) variables
//-----------------------------------------------------------------------------
VOID    AppPause( BOOL );
HRESULT App_InitDeviceObjects( LPDIRECT3DDEVICE3, LPDIRECT3DVIEWPORT3 );
VOID    App_DeleteDeviceObjects( LPDIRECT3DDEVICE3, LPDIRECT3DVIEWPORT3 );



//-----------------------------------------------------------------------------
// Name: App_OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
HRESULT App_OneTimeSceneInit( HWND hWnd, LPDIRECT3DDEVICE3 pd3dDevice,
                             LPDIRECT3DVIEWPORT3 pvViewport )
{
  
  // Create some textures
  D3DTextr_CreateTexture( g_textureFile0[0],  0);
  for (int i=1;i<=16;i++){
	  D3DTextr_CreateTexture( g_textureFile0[i],  0);
  }
  
  
  //create the landscape object
  g_pLandscape=new MyLandscape(30,30);

  //create the explosion object
  g_pExplosion=new Explosion;

  g_pExplosion2=new Explosion;
	g_pExplosion2->x=-10;
	g_pExplosion2->z=-70.f;
  

  // Initialize the device-dependant objects, such as materials and lights.
  // Note: this is in a separate function so that device changes can be
  // handled easily.
  return App_InitDeviceObjects( pd3dDevice, pvViewport );
}

#define FLOAT_PI                  3.14159265358979323846f


/////////////////////////////////////////////////////
// Static variable for camera interface
/////////////////////////////////////////////////////

static float vrad=70.0f;
static float ang=-FLOAT_PI/2.0f;
static float elev=18.0;

static float pitch=0;
static float yaw=0.0f;
static float roll=0.0f;
static float move=0;
static float moveLeft=0;
static float moveUp=0;
static D3DVECTOR Position(0,10.0f,-90.0f);

void HandleMouse(int dx,int dy,int button){

	if (button==0)
		return;
	if (button==1)//lbutton
	{
		if (abs(dx)>abs(dy)) dy=0;
		else if (abs(dy)>abs(dx)) dx=0;

		//x is yaw
		yaw-=0.1f*tan(dx/20.0f);//*.001;
		//y is pitch
		pitch-=0.1f*tan(dy/20.0f);//*.001;
	}
	else if (button==2)//rbutton
	{
		if (abs(dx)>abs(dy)) dy=0;
		else if (abs(dy)>abs(dx)) dx=0;
		//x is roll
		roll+=dx*.01;
		//y is move
		move+=-dy;
	}

}
void HandleKey(int vKey){
	switch (vKey){
	case VK_UP:
		move+=0.5f;
		break;
	case VK_DOWN:
		move-=0.5f;
		break;
	case VK_LEFT:
		moveLeft-=0.5f;
		break;
	case VK_RIGHT:
		moveLeft+=0.5f;
		break;
	case 'A':
		moveUp+=0.5f;
		break;
	case 'Z':
		moveUp-=0.5f;
		break;

	case 0xbe://dot
		roll-=FLOAT_PI/120.0f;
		break;
	case 0xbc://comma
		roll+=FLOAT_PI/120.0f;
		break;
	case 'W':
		bToggleWireframe=TRUE;
		break;
	case 'T':
		bTexture=!bTexture;
		break;
	case '1':
		g_ExpType=EXP_VOLUMETRICWORLDSPACE;
		break;
	case '2':
		g_ExpType=EXP_VOLUMETRIC;
		break;
	case '3':
		g_ExpType=EXP_BILLBOARD;
		break;
	case '0':
		//g_bTwoExplosions = !g_bTwoExplosions;
		break;
	case 'P':
		if (g_pExplosion->inc==0.0f)
		{
			g_pExplosion->inc=0.5f;
			g_pExplosion2->inc=0.5f;
		}
		else
		{
			g_pExplosion->inc=0.0f;
			g_pExplosion2->inc=0.0f;
		}
		break;
	case 'K':
		if (g_fZBias>0.0f)
			g_fZBias-= 1.0f/65536.0f;
		break;
		
	case 'L':
		if (g_fZBias<1.0f)
			g_fZBias+= 1.0f/65536.0f;
		break;
	case 'F':
		if (g_FilterMode == D3DTFG_LINEAR)
			g_FilterMode        = D3DTFG_POINT;
		else
			g_FilterMode        = D3DTFG_LINEAR;

		break;

	case 'B':
		g_bRenderBB = ! g_bRenderBB;
		break;
	case VK_SPACE:
		if (g_ExpType==EXP_BILLBOARD)
			g_ExpType=EXP_VOLUMETRIC;

		else if (g_ExpType==EXP_VOLUMETRIC)
			g_ExpType=EXP_VOLUMETRICWORLDSPACE;

		else if (g_ExpType==EXP_VOLUMETRICWORLDSPACE)
			g_ExpType=EXP_BILLBOARD;//
		//g_ExpType=EXP_VOLUMETRICNOTEXTURE;

		else if (g_ExpType==EXP_VOLUMETRICNOTEXTURE)
			g_ExpType=EXP_BILLBOARD;
		
		break;
	}
}

void SetCameraMatrix(float &pitch,float &yaw,float &roll,D3DVECTOR *pos,float &move,D3DMATRIX &mat)
{
	D3DVECTOR vView(mat._13,mat._23,mat._33),vUp(mat._12,mat._22,mat._32),
		vLat(mat._11,mat._21,mat._31);
	D3DVECTOR vNewView(0,0,1),vNewUp(0,1,0),vNewLat(1,0,0);
	D3DMATRIX tMat,res;
	float w;

	D3DUtil_SetIdentityMatrix(tMat);
	SWAP(mat._12 ,mat._21,float);   SWAP(mat._13 ,mat._31,float);
    SWAP(mat._23 ,mat._32,float);
	mat._41=mat._42=mat._43=0;
	if (pitch!=0)
	{
		D3DUtil_SetRotateXMatrix(tMat,pitch);
		pitch=0;
	}
	else if (yaw!=0)
	{

		D3DUtil_SetRotateYMatrix(tMat,yaw);
		yaw=0;
	}
	else if (roll!=0)
	{
		D3DUtil_SetRotateZMatrix(tMat,roll);
		roll=0;
	}
	D3DMath_MatrixMultiply(res,mat,tMat);
	mat=res;
    
    SWAP(mat._12 ,mat._21,float);   SWAP(mat._13 ,mat._31,float);
    SWAP(mat._23 ,mat._32,float);

	if (move!=0)
	{
		pos->x+=move*mat._13;
		pos->y+=move*mat._23;
		pos->z+=move*mat._33;
	}

	if (moveLeft!=0)
	{
		pos->x+=moveLeft*mat._11;
		pos->y+=moveLeft*mat._21;
		pos->z+=moveLeft*mat._31;
	}

	if (moveUp!=0)
	{
		pos->x+=moveUp*mat._12;
		pos->y+=moveUp*mat._22;
		pos->z+=moveUp*mat._32;
	}
    // Do the translation values (rotations are still about the eyepoint)
    mat._41 = - (pos->x*mat._11 + pos->y*mat._21 + pos->z*mat._31);
    mat._42 = - (pos->x*mat._12 + pos->y*mat._22 + pos->z*mat._32);
    mat._43 = - (pos->x*mat._13 + pos->y*mat._23 + pos->z*mat._33);

	moveLeft=moveUp=move=0;

}
//-----------------------------------------------------------------------------
// Name: App_FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
HRESULT App_FrameMove( LPDIRECT3DDEVICE3 pd3dDevice, FLOAT fTimeKey )
{
	//Do some calculations to move the camera.

	D3DVECTOR vEyePt    = D3DVECTOR( vrad*cos(ang), elev,  vrad*sin(ang)-60.);
    D3DVECTOR vLookatPt = D3DVECTOR( 0, 0,   -60  );
    D3DVECTOR vUpVec    = D3DVECTOR( 0, 1,   0  );

    SetCameraMatrix(pitch,yaw,roll,&Position,move,g_matView);
    //D3DUtil_SetViewMatrix( g_matView, vEyePt, vLookatPt, vUpVec );
	pd3dDevice->SetTransform( D3DTRANSFORMSTATE_VIEW,       &g_matView );
	
    return DD_OK;
}


//-----------------------------------------------------------------------------
// Name: App_Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
HRESULT App_Render( LPDIRECT3DDEVICE3 pd3dDevice, 
                    LPDIRECT3DVIEWPORT3 pvViewport,                    D3DRECT* prcViewportRect )
{
	//check to see if we need to change fillmode (debug use only)
	if (bToggleWireframe){
		bWireframe=!bWireframe;
		pd3dDevice->SetRenderState(D3DRENDERSTATE_FILLMODE,(bWireframe)?D3DFILL_WIREFRAME:D3DFILL_SOLID);
		bToggleWireframe=FALSE;
	}
	pd3dDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, g_FilterMode);
	pd3dDevice->SetTextureStageState(0, D3DTSS_MINFILTER, g_FilterMode );

    //Clear the viewport
    pvViewport->Clear( 1UL, prcViewportRect, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER );

    // Begin the scene 
    if( SUCCEEDED( pd3dDevice->BeginScene() ) )
    {
		
		/////////////////////////////////////////////////////////////////////
		//                   First Pass
		/////////////////////////////////////////////////////////////////////
		//set the landscape texture
		pd3dDevice->SetTexture( 0, D3DTextr_GetTexture(g_textureFile0[0]) );

		//render the landscape 
		g_pLandscape->Render(pd3dDevice);
		

		/////////////////////////////////////////////////////////////////////
		//                   Second Pass
		/////////////////////////////////////////////////////////////////////

		g_pExplosion->IncrementRadius();
		if (g_bTwoExplosions)
			g_pExplosion2->IncrementRadius();


		if (bTexture)
		{
			//cycle through the 16 texture frames
			int texindex=(int)(g_pExplosion->radius)/2+1;
			pd3dDevice->SetTexture( 0, D3DTextr_GetTexture(g_textureFile0[texindex]) );
		}
		else
		{
			pd3dDevice->SetTexture( 0, NULL );
		}
		

		//set to CLAMP to avoid artefacts - not necessary for renderZBias due to clipping
		pd3dDevice->SetTextureStageState(0,D3DTSS_ADDRESS,D3DTADDRESS_CLAMP);


		//enable alpha blending for second pass
		pd3dDevice->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE,1);

		//Disable perspective correction so that projected texture mapping looks correct on explosion
		pd3dDevice->SetRenderState(D3DRENDERSTATE_TEXTUREPERSPECTIVE,0);
		
		//render the explosion
		if (g_ExpType==EXP_VOLUMETRICWORLDSPACE||g_ExpType==EXP_VOLUMETRICNOTEXTURE)
		{			
			
			g_pExplosion->RenderZBias(pd3dDevice,g_pLandscape,&g_matView);
			if (g_bTwoExplosions)
				g_pExplosion2->RenderZBias(pd3dDevice,g_pLandscape,&g_matView);

		}
		else if (g_ExpType==EXP_VOLUMETRIC)
		{
			g_pExplosion->Render(pd3dDevice,g_pLandscape,&g_matView);
			if (g_bTwoExplosions)
				g_pExplosion2->Render(pd3dDevice,g_pLandscape,&g_matView);

		}
		else
		{
			g_pExplosion->RenderB(pd3dDevice,g_pLandscape,&g_matView);
			if (g_bTwoExplosions)
				g_pExplosion2->RenderB(pd3dDevice,g_pLandscape,&g_matView);

		}


		/////////////////////////////////////////////////////////////////////
		//                   Reset States
		/////////////////////////////////////////////////////////////////////
		//set renderstates back to normal
		pd3dDevice->SetRenderState(D3DRENDERSTATE_TEXTUREPERSPECTIVE,1);
		pd3dDevice->SetTextureStageState(0,D3DTSS_ADDRESS,D3DTADDRESS_WRAP);
		pd3dDevice->SetRenderState(D3DRENDERSTATE_ZFUNC,D3DCMP_LESSEQUAL);
		pd3dDevice->SetRenderState(D3DRENDERSTATE_ZWRITEENABLE,1);
		pd3dDevice->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE,0);

		// End the scene.
		pd3dDevice->EndScene();
    }
   

    return DD_OK;
}




//-----------------------------------------------------------------------------
// Name: App_InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
HRESULT App_InitDeviceObjects( LPDIRECT3DDEVICE3 pd3dDevice,
                               LPDIRECT3DVIEWPORT3 pvViewport )
{
    // Check parameters
    if( NULL==pd3dDevice || NULL==pvViewport )
        return DDERR_INVALIDPARAMS;

    LPDIRECT3D3 pD3D;
    if( FAILED( pd3dDevice->GetDirect3D( &pD3D ) ) )
        return E_FAIL;
    pD3D->Release();

    //Create the background material
    if( FAILED( pD3D->CreateMaterial( &g_pmtrlBackgroundMtrl, NULL) ) )
        return E_FAIL;

    HRESULT           hr;
    D3DMATERIAL       mtrl;
    D3DMATERIALHANDLE hmtrl;

	D3DUtil_InitMaterial( mtrl, 0, 0, 0 );
    g_pmtrlBackgroundMtrl->SetMaterial( &mtrl );
    g_pmtrlBackgroundMtrl->GetHandle( pd3dDevice, &hmtrl );
    pvViewport->SetBackground( hmtrl );


    // Create and set up the object material
    if( FAILED( hr = pD3D->CreateMaterial( &g_pmtrlObjectMtrl, NULL ) ) )
        return E_FAIL;

    D3DUtil_InitMaterial( mtrl, 0.5f, 0.5f, 0.5f );
    mtrl.power = 40.0f;
    g_pmtrlObjectMtrl->SetMaterial( &mtrl );
    g_pmtrlObjectMtrl->GetHandle( pd3dDevice, &hmtrl );
 
    pd3dDevice->SetLightState(  D3DLIGHTSTATE_MATERIAL, hmtrl );
    pd3dDevice->SetLightState(  D3DLIGHTSTATE_AMBIENT,  0x40404040 );

    D3DTextr_RestoreAllTextures( pd3dDevice, DDBD_32 );


	// Set up the light
    if( FAILED( pD3D->CreateLight( &g_pLight, NULL ) ) )
        return E_FAIL;
	D3DLIGHT light;
	D3DUtil_InitLight( light, D3DLIGHT_DIRECTIONAL, 0.0f, -10.0f, -10.0f );
	
	g_pLight->SetLight( &light );
    pvViewport->AddLight( g_pLight );

    // Set the transform matrices

	D3DVECTOR vEyePt    = D3DVECTOR( 0.0, 4.,   10. );
    D3DVECTOR vLookatPt = D3DVECTOR( 0, 0,   1  );
    D3DVECTOR vUpVec    = D3DVECTOR( 0, 1,   0  );
    
    D3DMATRIX matWorld, matProj;

    D3DUtil_SetIdentityMatrix( matWorld );
    D3DUtil_SetViewMatrix( g_matView, Position, vLookatPt, vUpVec );
 
	D3DUtil_SetProjectionMatrix2( matProj, 90.0f, 1.0f, 1000.0f );

    pd3dDevice->SetTransform( D3DTRANSFORMSTATE_WORLD,      &matWorld );
	
    pd3dDevice->SetTransform( D3DTRANSFORMSTATE_VIEW,       &g_matView );
    pd3dDevice->SetTransform( D3DTRANSFORMSTATE_PROJECTION, &matProj );

    pd3dDevice->SetRenderState( D3DRENDERSTATE_ZENABLE, D3DZB_TRUE );
	pd3dDevice->SetRenderState( D3DRENDERSTATE_DITHERENABLE, 1 );
	pd3dDevice->SetRenderState(D3DRENDERSTATE_SHADEMODE,D3DSHADE_GOURAUD);
    pd3dDevice->SetRenderState(D3DRENDERSTATE_CULLMODE,D3DCULL_CCW);

	pd3dDevice->SetRenderState(D3DRENDERSTATE_SRCBLEND,D3DBLEND_ONE);
	pd3dDevice->SetRenderState(D3DRENDERSTATE_DESTBLEND,D3DBLEND_ONE);

	pd3dDevice->SetRenderState(D3DRENDERSTATE_FILLMODE,D3DFILL_SOLID);
    pd3dDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_LINEAR);
	pd3dDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFG_LINEAR );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: App_FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
HRESULT App_FinalCleanup( LPDIRECT3DDEVICE3 pd3dDevice, 
                          LPDIRECT3DVIEWPORT3 pvViewport)
{
    App_DeleteDeviceObjects( pd3dDevice, pvViewport );
	    
	SAFE_DELETE( g_pLandscape);
	SAFE_DELETE( g_pExplosion);
	SAFE_DELETE( g_pExplosion2);
    return DD_OK;
}



//-----------------------------------------------------------------------------
// Name: App_DeleteDeviceObjects()
// Desc: Called when the app is exitting, or the device is being changed,
//       this function deletes any device dependant objects.
//-----------------------------------------------------------------------------
VOID App_DeleteDeviceObjects( LPDIRECT3DDEVICE3 pd3dDevice, 
                              LPDIRECT3DVIEWPORT3 pvViewport)
{
    D3DTextr_InvalidateAllTextures();

    SAFE_RELEASE( g_pLight );
    SAFE_RELEASE( g_pmtrlObjectMtrl );
    SAFE_RELEASE( g_pmtrlBackgroundMtrl );
}




//----------------------------------------------------------------------------
// Name: App_RestoreSurfaces
// Desc: Restores any previously lost surfaces. Must do this for all surfaces
//       (including textures) that the app created.
//----------------------------------------------------------------------------
HRESULT App_RestoreSurfaces( )
{
    D3DTextr_RestoreAllTextureSurfaces();
    return D3D_OK;
}




//-----------------------------------------------------------------------------
// Name: App_ConfirmDevice()
// Desc: Called during device intialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
HRESULT App_ConfirmDevice( DDCAPS* pddDriverCaps,
						   D3DDEVICEDESC* pd3dDeviceDesc )
{

  return S_OK;

}






